@@ -86,14 +86,21 @@ export class SerialHubPort {
     }
   }
 
-  writeToStream(data: ArrayBufferView[] | ArrayBuffer[]): void {
-    if (this.writer) {
-      data.forEach(async (d: any) => {
-        //Anonymous function is ASYNC so it can AWAIT the write() call below
-        console.log('[WRITE]', d);
-        await this.writer?.write(d); // AWAIT in sequence, to avoid parallel promises
-      });
+  writeToStream(data: ArrayBufferView[] | ArrayBuffer[]): number {
+    if (!this.writer) {
+      throw new TypeError('Stream not open');
     }
+    data.forEach(async (d: ArrayBufferView | ArrayBuffer) => {
+      //Anonymous function is ASYNC so it can AWAIT the write() call below
+      console.log('[WRITE]', d, d.byteLength);
+      await this.writer?.write(d); // AWAIT in sequence, to avoid parallel promises
+    });
+    let nWritten = 0;
+    for (const d of data) {
+      nWritten += d.byteLength; //TODO: Research if offset needs to be considered???
+    }
+    console.log('[WROTE]', nWritten);
+    return nWritten;
   }
 
   async readLoop(cbRead: (theVAL: any) => void): Promise<void> {
