@@ -38,11 +38,11 @@ export class SerialHubModel extends DOMWidgetModel {
       status: 'Initializing...',
       value: 'Loading...',
       request_options: {},
-      serial_options: {},
-      pkt_recv_front: 0,
-      pkt_recv_back: 0,
-      pkt_send_front: 0,
-      pkt_send_back: 0
+      serial_options: {}, //Rely initial backend sync with frontend for real default
+      pkt_recv_front: [0, 0],
+      pkt_send_front: [0, 0],
+      pkt_recv_back: [0, 0],
+      pkt_send_back: [0, 0]
     };
   }
 
@@ -66,8 +66,9 @@ export class SerialHubModel extends DOMWidgetModel {
 
 export class SerialHubView extends DOMWidgetView {
   protected _el_status: HTMLButtonElement | null = null;
-  protected _el_stats: HTMLSpanElement | null = null;
+  protected _el_stats: HTMLPreElement | null = null;
   protected _el_value: HTMLPreElement | null = null;
+
   _shp: SerialHubPort | null = null;
 
   render(): this {
@@ -77,7 +78,7 @@ export class SerialHubView extends DOMWidgetView {
     /* Create a couple sub-Elements for our custom widget */
     this._el_status = window.document.createElement('button');
     this._el_status.classList.add('xx-serialhub-status');
-    this._el_stats = window.document.createElement('span');
+    this._el_stats = window.document.createElement('pre');
     this._el_stats.classList.add('xx-serialhub-stats');
     this._el_value = window.document.createElement('pre');
     this._el_value.classList.add('xx-serialhub-value');
@@ -92,6 +93,7 @@ export class SerialHubView extends DOMWidgetView {
     this.changed_status();
     this.changed_value();
     this.changed_stats();
+    this.update_stats_title();
     this.model.on('change:status', this.changed_status, this);
     this.model.on('change:value', this.changed_value, this);
     this.model.on('change:request_options', this.changed_request_options, this);
@@ -103,11 +105,10 @@ export class SerialHubView extends DOMWidgetView {
 
     this.model.on('msg:custom', this.msg_custom, this);
 
-    this.model.set('is_supported', SerialHubPort.isSupported());
-    this.model.set(
-      'status',
-      SerialHubPort.isSupported() ? 'Supported' : 'Unsupported'
-    );
+    const supported: boolean = SerialHubPort.isSupported();
+    this.model.set('is_supported', supported);
+    this.model.set('status', supported ? 'Supported' : 'Unsupported');
+
     this.touch();
     return this;
   }
@@ -154,26 +155,32 @@ export class SerialHubView extends DOMWidgetView {
   changed_stats(): void {
     if (this._el_stats) {
       let stats = '';
-      stats += ' Rf:' + this.model.get('pkt_recv_front');
-      stats += ' Rb:' + this.model.get('pkt_recv_back');
-      stats += ' Sf:' + this.model.get('pkt_send_front');
-      stats += ' Sb:' + this.model.get('pkt_send_back');
+      stats += ' RecvF:' + this.model.get('pkt_recv_front');
+      stats += ' SendF:' + this.model.get('pkt_send_front');
+      stats += '  (Front-End)\r\n';
+      stats += ' RecvB:' + this.model.get('pkt_recv_back');
+      stats += ' SendB:' + this.model.get('pkt_send_back');
+      stats += '  (Back-End)';
       this._el_stats.textContent = stats;
     }
   }
 
+  protected inc_stats_tuple(key: string, nBytes: number, nPackets = 1): void {
+    const [oByt, oPkt] = this.model.get(key);
+    this.model.set(key, [oByt + nBytes, oPkt + nPackets]);
+  }
+
   cb_read(this: SerialHubView, value: ArrayBuffer): void {
-    console.log('DATA-IN', value);
+    //console.log('DATA-IN', value);
     try {
       this.model.send({ type: 'RECV' }, {}, [value]);
     } catch (e) {
-      console.log('Failed to send serial data to backend.', e);
+      console.log('FAILED send of serial data to backend.', e);
       //TODO: Shutdown the reader & connection on fatal errors
       throw e; //Rethrow exception
     }
     //Only increment statistics if send() was successful
-    const cnt: number = this.model.get('pkt_recv_front') + 1;
-    this.model.set('pkt_recv_front', cnt);
+    this.inc_stats_tuple('pkt_recv_front', value.byteLength);
   }
 
   cb_connect(this: SerialHubView): void {
@@ -191,22 +198,23 @@ export class SerialHubView extends DOMWidgetView {
     //const reqOpts = { filters: [{usbVendorId: 0x2047}] }; // TI proper ; unused 0x0451 for "TUSB2046 Hub"
     //const serOpts = { baudRate: 115200 };
     const [reqOpts, serOpts] = this.get_port_options(); //Unpack options to local vars
-    console.log('Connect with options...', reqOpts, serOpts);
-    this._shp.connect(reqOpts, serOpts).then((): void => {
-      this.cb_connect();
-    });
-    console.log('DONE click', this._shp);
+    console.log('CONNECT options', reqOpts, serOpts);
+    this._shp.connect(reqOpts, serOpts).then(
+      (): void => {
+        this.model.set('status', 'Connected');
+        this.cb_connect();
+      },
+      (reason: any): void => {
+        this.model.set('status', 'Disconnected');
+      }
+    );
+    console.log('click_status DONE', this._shp);
   }
 
   click_value(this: SerialHubView, ev: MouseEvent): void {
     if (!this || !this.model) {
       return;
     }
-    this.model.send({ type: 'text', text: 'VALUE-6\n' }, {}, []);
-    const encoder = new TextEncoder();
-    const theData = encoder.encode('6');
-    this._shp?.writeToStream([theData]);
-    this.model.set('pkt_send_front', this.model.get('pkt_send_front') + 1);
   }
 
   msg_custom(this: SerialHubView, mData: Dict<any>, mBuffs: DataView[]): void {
@@ -214,13 +222,10 @@ export class SerialHubView extends DOMWidgetView {
     const msgType = mData['type'];
     if (msgType === 'SEND') {
       console.log('MSG-SEND', mBuffs);
-      this._shp?.writeToStream(mBuffs);
-      this.model.set('pkt_send_front', this.model.get('pkt_send_front') + 1);
-    } else if (msgType === 'SEND2') {
-      const encoder = new TextEncoder();
-      const theData = encoder.encode(mData['text']);
-      this._shp?.writeToStream([theData]);
-      this.model.set('pkt_send_front', this.model.get('pkt_send_front') + 1);
+      if (this._shp) {
+        const nWritten: number = this._shp.writeToStream(mBuffs);
+        this.inc_stats_tuple('pkt_send_front', nWritten);
+      }
     } else {
       console.log('UNKNOWN MESSAGE: ', msgType, mData, mBuffs);
     }
